/*
 * Asterisk -- A telephony toolkit for Linux.
 *
 * Excute application -- runs popen for you
 *
 * Copyright (C) 2005 Matt O'Gorman.  Rights assigned to Digium.
 * readpipe info - Runs a command and returns what command returns.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <asterisk/file.h>
#include <asterisk/logger.h>
#include <asterisk/options.h>
#include <asterisk/channel.h>
#include <asterisk/pbx.h>
#include <asterisk/module.h>

static char *tdesc = "Stores output of command into a variable";

static char *app_readpipe = "ReadPipe";

static char *readpipe_synopsis = "ReadPipe(varname=program,lines)";

static char *readpipe_descrip =
"ReadPipe(varname=program,lines)\n"
"  Varname - Result stored here.\n"
"  Program - The name of the program to run, optionally enclosed in quotes.\n"
"  Lines   - Maximum lines to capture.  Defaults to infinite.\n";

STANDARD_LOCAL_USER;

LOCAL_USER_DECL;

static char *readpipe_popen(int len, char *prog)
{
	FILE *ast_command;
	char line[2048];
	int i=1;
	int flag=0;
	char *output=NULL;

	ast_command = popen(prog, "r");

	if (!ast_command) {
		ast_log(LOG_ERROR, "Cannot open pipe\n");
		return NULL;
	}

	while (fgets(line, sizeof(line), ast_command) && (i < len || len==0)) {
		if (flag == 0) {
			output = (char *)malloc((strlen(line) + 1) * sizeof(char));
			strcpy(output, line);
			flag = 1;
		} else if (output) { 
			output = (char *) realloc(output, (strlen(output) + strlen(line) + 1) * sizeof(char));
			strcat(output, line);
			i++;
		} else {
			ast_log(LOG_ERROR, "Out of memory\n");
		}
	}

	pclose(ast_command);
	return output;
}

static int readpipe_exec(struct ast_channel *chan, void *data)
{
	int res=0;
	struct localuser *u;
	char *s, *varname=NULL, *program=NULL, *length=NULL, *returnvar=NULL;
	int arglen;
	int len=0;

	LOCAL_USER_ADD(u);

	s = ast_strdupa(data);
	if (!s) {
		ast_log(LOG_ERROR, "Out of memory\n");
		return -1;
	}

	varname = strsep(&s, "=");
	if (s && *s == '"') {
		s++;
		program = strsep(&s, "\"");
		if (s && *s == '|') {
			s++;
		}
	} else {
		program = strsep(&s, "|");
	}
	length = s;

	if (!varname || !program) {
		ast_log(LOG_ERROR, "No program specified!\n");
		return -1;
	}

	if (length) {
		if (!sscanf(length, "%d", &len)) {
			ast_log(LOG_WARNING, "'%s' is not a number, defaulting length to max\n", length);
		}
	}

	returnvar = readpipe_popen(len, program);
	pbx_builtin_setvar_helper(chan, varname, returnvar);
	free(returnvar);

	LOCAL_USER_REMOVE(u);
	return res;
}


int unload_module(void)
{
	STANDARD_HANGUP_LOCALUSERS;
	return ast_unregister_application(app_readpipe);
}

int load_module(void)
{
	return ast_register_application(app_readpipe, readpipe_exec, readpipe_synopsis, readpipe_descrip);
}

char *description(void)
{
	return tdesc;
}

int usecount(void)
{
	int res;
	STANDARD_USECOUNT(res);
	return res;
}

char *key()
{
	return ASTERISK_GPL_KEY;
}
